/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/


#include "test.priv.h"

#include <sys/time.h>
#include <time.h>
#include <signal.h>
#include <string.h>
#include "novas.h"
#include "ang_tools.h"


#define YBASE	 2 // 10 2
#define XBASE	15 // 10 15
#define XLENGTH	63 // 54 56
#define YDEPTH	5

#define NEG 11

static int disp[12] =
{
    075557, 011111, 071747, 071717, 055711,
    074717, 074757, 071111, 075757, 075717,
    002020, 000300 
};

/*
static int disp[12] =
{
    075757, 075757, 075757, 075757, 075757,
    075757, 075757, 075757, 075757, 075757, 
    002020, 000700
};
*/

struct digital_clock {
	int x;
	int y;
	long older[6];
	long next[6];
	long newer[6];
	long mask;
};


static int sigtermed = 0;
static bool redirected = FALSE;
static bool hascolor = FALSE;


static RETSIGTYPE sighndl(int signo)
{
	signal(signo, sighndl);
	sigtermed = signo;
	if (redirected) {
		endwin();
		exit(EXIT_FAILURE);
	}
}


static void drawbox(int x, int y, int xl, int yl)
{
	chtype bottom[xl + 1];
	int n;
	
	if (hascolor)
		attrset(COLOR_PAIR(3));
	
	mvaddch(y - 1, x - 1, ACS_ULCORNER);
	hline(ACS_HLINE, xl);
	mvaddch(y - 1, x + xl, ACS_URCORNER);
	mvaddch(y + yl, x - 1, ACS_LLCORNER);
	mvinchnstr(y + yl, x, bottom, xl);
	for (n = 0; n < xl; n++)
		bottom[n] = ACS_HLINE | (bottom[n] & (A_ATTRIBUTES | A_COLOR));
	mvaddchnstr(y + yl, x, bottom, xl);
	mvaddch(y + yl, x + xl, ACS_LRCORNER);

	move(y, x - 1);
	vline(ACS_VLINE, yl);
	
	move(y, x + xl);
	vline(ACS_VLINE, yl);

	if (hascolor)
		attrset(COLOR_PAIR(2));
}



static void standt(int on)
{
	if (on) {
		if (hascolor) {
			attron(COLOR_PAIR(1));
		} else {
			attron(A_STANDOUT);
		}
	} else {
		if (hascolor) {
			attron(COLOR_PAIR(2));
		} else {
			attroff(A_STANDOUT);
		}
	}
}


static void set(struct digital_clock *d, int t, int n)
{
	long i, m;
	
	/*             8421 8421 */
	m = 7 << n; /* 0000|0111 */
	for (i = 0; i < 5; i++) {
		d->next[i] |= ((disp[t] >> ((4 - i) * 3)) & 07) << n;
		d->mask |= ((d->next[i]) ^ (d->older[i])) & m;
	}
	if ((d->mask) & m)
		d->mask |= m;
}


int dig_init(struct digital_clock *d, int x, int y)
{
	int i;
	
	for (i = 0; i < 5; i++)
		d->older[i] = d->newer[i] = d->next[i] = 0;
	d->x = x;
	d->y = y;
	d->mask = 0;

	return 0;
}

	

int dig_display(struct digital_clock *d,
		int sec, int min, int hour)
{
	int i, j, s, k;
	long t, a;

	d->mask = 0;
	set(d, sec % 10, 0);
	set(d, sec / 10, 4);
	set(d, min % 10, 10);
	set(d, min / 10, 14);

	if (hour < 0) {
		hour *= -1;
		set(d, NEG , 28);		
	}
	if ( hour > 99) {
		set(d, hour / 100, 28);
		hour -= 100;
	}
	set(d, hour % 10, 20);
	set(d, hour / 10, 24);

	set(d, 10, 7);
	set(d, 10, 17);

	for (k = 0; k < 6; k++) {
		d->newer[k] = (d->newer[k] & ~(d->mask)) | 
			(d->next[k] & (d->mask));
		d->next[k] = 0;
		for (s = 1; s >= 0; s--) {
			standt(s);
			for (i = 0; i < 5; i++) {
				if ((a = (d->newer[i] ^ d->older[i]) & 
				     (s ? d->newer : d->older)[i])
				    != 0) {
					for (j = 0, t = 1 << 30/*26*/; t; 
					     t >>= 1, j++) {
						if (a & t) {
							if (!(a & (t << 1))) {
								move(d->y + i, d->x + 2 * j);
							}
							addstr("  ");
						}
					}
		    }
				if (!s) {
					d->older[i] = d->newer[i];
				}
			}
		}
	}
	drawbox(d->x, d->y, XLENGTH, YDEPTH );

	return 0;
}


static void usage(void)
{
    static const char *msg[] =
    {
	"Usage: sidt [options] [count]"
	,""
	,"Options:"
	,"  -n  redirect input to /dev/null"
	,""
	,"  -l  longitude"
	,""
	,"If you specify a count, gdc runs for that number of seconds"
    };
    unsigned j;
    for (j = 0; j < SIZEOF(msg); j++)
	fprintf(stderr, "%s\n", msg[j]);
    exit(EXIT_FAILURE);
}


int main(int argc, char *argv[])
{
	struct tm *tm;
	struct timeval now/*, nowl*/;
	int   k;
	int count = 0;
	FILE *ofp = stdout;
	FILE *ifp = stdin;
	struct digital_clock d0, d1, d2;
	double tjd;
	double gst;
	double lst;
	double s0, s1;
	int h,m,s;
	double ra = 0;
	char ra_str[16];
	double dec = 0;
	char dec_str[16];
	double dtmp;
	double longitud = -57.9322995; /* - (57.0 +(55.0 + 56.2782/60.0)/60.0) */
	char long_str[16];
	char buf[80];
	signal(SIGINT, sighndl);
	signal(SIGTERM, sighndl);
	signal(SIGKILL, sighndl);
	
	while ((k = getopt(argc, argv, "nl:r:d:")) != EOF) {
		switch (k) {
		case 'n':
			ifp = fopen("/dev/null", "r");
			redirected = TRUE;
			break;
		case 'l':
			//longitud = atof(optarg);
			longitud = strdeg2double(optarg);
			break;
		case 'r':
			ra = strdeg2double(optarg);
			break;
		case 'd':
			dec = strdeg2double(optarg);
			break;
		default:
			usage();
		}
	}
	if (optind < argc) {
		count = atoi(argv[optind++]);
	}
	if (optind < argc)
	usage();
	
	if (redirected) {
		char *name = getenv("TERM");
		if (name == 0
		    || newterm(name, ofp, ifp) == 0) {
			fprintf(stderr, "cannot open terminal\n");
			exit(EXIT_FAILURE);
		}
		
	} else {
		initscr();
	}
	cbreak();
	noecho();
	nodelay(stdscr, 1);
	curs_set(0);
	
	hascolor = has_colors();
	
	if (hascolor) {
		int bg = COLOR_BLACK;
		start_color();
#if HAVE_USE_DEFAULT_COLORS
		if (use_default_colors() == OK)
			bg = -1;
#endif
		init_pair(1, COLOR_BLACK, COLOR_RED /*COLOR_GREEN */);
		init_pair(2, COLOR_RED /*COLOR_GREEN*/, bg);
		init_pair(3, COLOR_WHITE /*COLOR_GREEN*/, bg);
		attrset(COLOR_PAIR(2));
	}

	double2gms_str( long_str, longitud);
	double2gms_str( ra_str, ra);
	double2gms_str( dec_str, dec);
/*****************/
//    count = 1;
/*****************/
	
 restart:
	dig_init(&d0, XBASE, YBASE);
	dig_init(&d1, XBASE, YBASE+YDEPTH+2);
	dig_init(&d2, XBASE, YBASE+(YDEPTH+2)*2);
	clear();
	
	mvaddstr(4, 1, "HA:");
	mvaddstr(4+YDEPTH+2, 1, "Dist. Pol. N:");
	mvaddstr(4+(YDEPTH+2)*2, 1, "Dist. Pol. S:");
	move(23,1);
	refresh();
	
	printf("q: quit|s: freez|space:continue");
	do {
		gettimeofday(&now, NULL);
		tm = gmtime( (time_t *) &now.tv_sec);
		tjd = julian_date((short int)(tm->tm_year + 1900), 
				  (short int)(tm->tm_mon + 1), 
				  (short int)tm->tm_mday, 
				  (double) tm->tm_hour + 
				  (((double)tm->tm_sec + 
				    ((double) now.tv_usec / 1000000.0)
					  )/60.0 + 
				   ((double)tm->tm_min))/60.0 
			);
		sidereal_time (tjd,0.0, 0.0, &gst);
		lst = mod24(gst + longitud / 15.0);
		double2gms(lst, &h, &m, &s);
		sprintf(buf,
			"ST: %02d:%02d:%02d JD: %13.5f RA: %s DEC: %s",
			h,m,s, tjd, ra_str, dec_str);
		mvaddstr(22, 1, buf);

		dtmp = mod24(lst - ra);
		double2gms(dtmp, &h, &m, &s);
		dig_display(&d0, s, m, h);

		dtmp = 90.0 - dec;
		double2gms(dtmp, &h, &m, &s);
		dig_display(&d1, s, m, h);

		dtmp = 90.0 + dec;
		double2gms(dtmp, &h, &m, &s);
		dig_display(&d2, s, m, h);

		(void) strcpy(buf, ctime(&now.tv_sec));
		mvaddstr(0, 1, buf);
		sprintf(buf, "Lon: %s", long_str);
		mvaddstr(0, 62, buf);

		s0 = (1.0 - (lst - (int) lst)) * 1000000.0;
		s1 = 1000000.0 - (double) now.tv_usec;
		if (s1 < s0) {
			s0 = s1;
		}

		refresh();
		
		usleep((unsigned long) (s0));
		switch (wgetch(stdscr)) {
		case 'q':
			count = 1;
			break;
		case 's':
			nodelay(stdscr, FALSE);
			break;
		case ' ':
			nodelay(stdscr, TRUE);
			break;
#ifdef KEY_RESIZE
		case KEY_RESIZE:
#endif
		case '?':
			goto restart;
		case ERR:
			if (sigtermed) {
				standend();
				endwin();
				fprintf(stderr, 
					"gdc terminated by signal %d\n", 
					sigtermed);
				return EXIT_FAILURE;
			}
			/* FALLTHRU */
			continue;
		default:
			count = 1;
			break;
		}
	} while (--count);
	move(23,1);
	refresh();
	printf("%-79s", " ");

	standend();
	endwin();
	return EXIT_SUCCESS;
}
